@@ -1,6 +1,7 @@
 package swarm
 
 import (
+	"errors"
 	"fmt"
 	"io"
 	"testing"
@@ -8,9 +9,7 @@ import (
 
 	"github.com/docker/cli/internal/test"
 	"github.com/docker/cli/internal/test/builders"
-	"github.com/docker/docker/api/types"
 	"github.com/docker/docker/api/types/swarm"
-	"github.com/pkg/errors"
 	"gotest.tools/v3/assert"
 	"gotest.tools/v3/golden"
 )
@@ -22,7 +21,7 @@ func TestSwarmUpdateErrors(t *testing.T) {
 		flags                 map[string]string
 		swarmInspectFunc      func() (swarm.Swarm, error)
 		swarmUpdateFunc       func(swarm swarm.Spec, flags swarm.UpdateFlags) error
-		swarmGetUnlockKeyFunc func() (types.SwarmUnlockKeyResponse, error)
+		swarmGetUnlockKeyFunc func() (swarm.UnlockKeyResponse, error)
 		expectedError         string
 	}{
 		{
@@ -36,7 +35,7 @@ func TestSwarmUpdateErrors(t *testing.T) {
 				flagTaskHistoryLimit: "10",
 			},
 			swarmInspectFunc: func() (swarm.Swarm, error) {
-				return swarm.Swarm{}, errors.Errorf("error inspecting the swarm")
+				return swarm.Swarm{}, errors.New("error inspecting the swarm")
 			},
 			expectedError: "error inspecting the swarm",
 		},
@@ -46,7 +45,7 @@ func TestSwarmUpdateErrors(t *testing.T) {
 				flagTaskHistoryLimit: "10",
 			},
 			swarmUpdateFunc: func(swarm swarm.Spec, flags swarm.UpdateFlags) error {
-				return errors.Errorf("error updating the swarm")
+				return errors.New("error updating the swarm")
 			},
 			expectedError: "error updating the swarm",
 		},
@@ -58,14 +57,13 @@ func TestSwarmUpdateErrors(t *testing.T) {
 			swarmInspectFunc: func() (swarm.Swarm, error) {
 				return *builders.Swarm(), nil
 			},
-			swarmGetUnlockKeyFunc: func() (types.SwarmUnlockKeyResponse, error) {
-				return types.SwarmUnlockKeyResponse{}, errors.Errorf("error getting unlock key")
+			swarmGetUnlockKeyFunc: func() (swarm.UnlockKeyResponse, error) {
+				return swarm.UnlockKeyResponse{}, errors.New("error getting unlock key")
 			},
 			expectedError: "error getting unlock key",
 		},
 	}
 	for _, tc := range testCases {
-		tc := tc
 		t.Run(tc.name, func(t *testing.T) {
 			cmd := newUpdateCommand(
 				test.NewFakeCli(&fakeClient{
@@ -73,12 +71,16 @@ func TestSwarmUpdateErrors(t *testing.T) {
 					swarmUpdateFunc:       tc.swarmUpdateFunc,
 					swarmGetUnlockKeyFunc: tc.swarmGetUnlockKeyFunc,
 				}))
-			cmd.SetArgs(tc.args)
-			for key, value := range tc.flags {
-				assert.Check(t, cmd.Flags().Set(key, value))
+			if tc.args == nil {
+				cmd.SetArgs([]string{})
+			} else {
+				cmd.SetArgs(tc.args)
 			}
 			cmd.SetOut(io.Discard)
 			cmd.SetErr(io.Discard)
+			for k, v := range tc.flags {
+				assert.Check(t, cmd.Flags().Set(k, v))
+			}
 			assert.ErrorContains(t, cmd.Execute(), tc.expectedError)
 		})
 	}
@@ -94,7 +96,7 @@ func TestSwarmUpdate(t *testing.T) {
 		flags                 map[string]string
 		swarmInspectFunc      func() (swarm.Swarm, error)
 		swarmUpdateFunc       func(swarm swarm.Spec, flags swarm.UpdateFlags) error
-		swarmGetUnlockKeyFunc func() (types.SwarmUnlockKeyResponse, error)
+		swarmGetUnlockKeyFunc func() (swarm.UnlockKeyResponse, error)
 	}{
 		{
 			name: "noargs",
@@ -115,33 +117,33 @@ func TestSwarmUpdate(t *testing.T) {
 			},
 			swarmUpdateFunc: func(swarm swarm.Spec, flags swarm.UpdateFlags) error {
 				if *swarm.Orchestration.TaskHistoryRetentionLimit != 10 {
-					return errors.Errorf("historyLimit not correctly set")
+					return errors.New("historyLimit not correctly set")
 				}
 				heartbeatDuration, err := time.ParseDuration("10s")
 				if err != nil {
 					return err
 				}
 				if swarm.Dispatcher.HeartbeatPeriod != heartbeatDuration {
-					return errors.Errorf("heartbeatPeriodLimit not correctly set")
+					return errors.New("heartbeatPeriodLimit not correctly set")
 				}
 				certExpiryDuration, err := time.ParseDuration("20s")
 				if err != nil {
 					return err
 				}
 				if swarm.CAConfig.NodeCertExpiry != certExpiryDuration {
-					return errors.Errorf("certExpiry not correctly set")
+					return errors.New("certExpiry not correctly set")
 				}
 				if len(swarm.CAConfig.ExternalCAs) != 1 || swarm.CAConfig.ExternalCAs[0].CACert != "trustroot" {
-					return errors.Errorf("externalCA not correctly set")
+					return errors.New("externalCA not correctly set")
 				}
 				if *swarm.Raft.KeepOldSnapshots != 10 {
-					return errors.Errorf("keepOldSnapshots not correctly set")
+					return errors.New("keepOldSnapshots not correctly set")
 				}
 				if swarm.Raft.SnapshotInterval != 100 {
-					return errors.Errorf("snapshotInterval not correctly set")
+					return errors.New("snapshotInterval not correctly set")
 				}
 				if !swarm.EncryptionConfig.AutoLockManagers {
-					return errors.Errorf("autolock not correctly set")
+					return errors.New("autolock not correctly set")
 				}
 				return nil
 			},
@@ -154,22 +156,21 @@ func TestSwarmUpdate(t *testing.T) {
 			},
 			swarmUpdateFunc: func(swarm swarm.Spec, flags swarm.UpdateFlags) error {
 				if *swarm.Orchestration.TaskHistoryRetentionLimit != 10 {
-					return errors.Errorf("historyLimit not correctly set")
+					return errors.New("historyLimit not correctly set")
 				}
 				return nil
 			},
 			swarmInspectFunc: func() (swarm.Swarm, error) {
 				return *builders.Swarm(), nil
 			},
-			swarmGetUnlockKeyFunc: func() (types.SwarmUnlockKeyResponse, error) {
-				return types.SwarmUnlockKeyResponse{
+			swarmGetUnlockKeyFunc: func() (swarm.UnlockKeyResponse, error) {
+				return swarm.UnlockKeyResponse{
 					UnlockKey: "unlock-key",
 				}, nil
 			},
 		},
 	}
 	for _, tc := range testCases {
-		tc := tc
 		t.Run(tc.name, func(t *testing.T) {
 			cli := test.NewFakeCli(&fakeClient{
 				swarmInspectFunc:      tc.swarmInspectFunc,
@@ -182,8 +183,8 @@ func TestSwarmUpdate(t *testing.T) {
 			} else {
 				cmd.SetArgs(tc.args)
 			}
-			for key, value := range tc.flags {
-				assert.Check(t, cmd.Flags().Set(key, value))
+			for k, v := range tc.flags {
+				assert.Check(t, cmd.Flags().Set(k, v))
 			}
 			cmd.SetOut(cli.OutBuffer())
 			assert.NilError(t, cmd.Execute())
