@@ -10,10 +10,9 @@ import (
 	"strings"
 	"testing"
 
-	"github.com/docker/docker/api/types"
+	cerrdefs "github.com/containerd/errdefs"
 	"github.com/docker/docker/api/types/filters"
 	"github.com/docker/docker/api/types/swarm"
-	"github.com/docker/docker/errdefs"
 	"gotest.tools/v3/assert"
 	is "gotest.tools/v3/assert/cmp"
 )
@@ -23,7 +22,7 @@ func TestSecretListUnsupported(t *testing.T) {
 		version: "1.24",
 		client:  &http.Client{},
 	}
-	_, err := client.SecretList(context.Background(), types.SecretListOptions{})
+	_, err := client.SecretList(context.Background(), swarm.SecretListOptions{})
 	assert.Check(t, is.Error(err, `"secret list" requires API version 1.25, but the Docker daemon API version is 1.24`))
 }
 
@@ -33,25 +32,25 @@ func TestSecretListError(t *testing.T) {
 		client:  newMockClient(errorMock(http.StatusInternalServerError, "Server error")),
 	}
 
-	_, err := client.SecretList(context.Background(), types.SecretListOptions{})
-	assert.Check(t, is.ErrorType(err, errdefs.IsSystem))
+	_, err := client.SecretList(context.Background(), swarm.SecretListOptions{})
+	assert.Check(t, is.ErrorType(err, cerrdefs.IsInternal))
 }
 
 func TestSecretList(t *testing.T) {
 	const expectedURL = "/v1.25/secrets"
 
 	listCases := []struct {
-		options             types.SecretListOptions
+		options             swarm.SecretListOptions
 		expectedQueryParams map[string]string
 	}{
 		{
-			options: types.SecretListOptions{},
+			options: swarm.SecretListOptions{},
 			expectedQueryParams: map[string]string{
 				"filters": "",
 			},
 		},
 		{
-			options: types.SecretListOptions{
+			options: swarm.SecretListOptions{
 				Filters: filters.NewArgs(
 					filters.Arg("label", "label1"),
 					filters.Arg("label", "label2"),
@@ -95,11 +94,7 @@ func TestSecretList(t *testing.T) {
 		}
 
 		secrets, err := client.SecretList(context.Background(), listCase.options)
-		if err != nil {
-			t.Fatal(err)
-		}
-		if len(secrets) != 2 {
-			t.Fatalf("expected 2 secrets, got %v", secrets)
-		}
+		assert.NilError(t, err)
+		assert.Check(t, is.Len(secrets, 2))
 	}
 }
