@@ -17,21 +17,23 @@ func (cli *Client) VolumeInspect(ctx context.Context, volumeID string) (volume.V
 
 // VolumeInspectWithRaw returns the information about a specific volume in the docker host and its raw representation
 func (cli *Client) VolumeInspectWithRaw(ctx context.Context, volumeID string) (volume.Volume, []byte, error) {
-	if volumeID == "" {
-		return volume.Volume{}, nil, objectNotFoundError{object: "volume", id: volumeID}
+	volumeID, err := trimID("volume", volumeID)
+	if err != nil {
+		return volume.Volume{}, nil, err
 	}
 
-	var vol volume.Volume
 	resp, err := cli.get(ctx, "/volumes/"+volumeID, nil, nil)
 	defer ensureReaderClosed(resp)
 	if err != nil {
-		return vol, nil, err
+		return volume.Volume{}, nil, err
 	}
 
-	body, err := io.ReadAll(resp.body)
+	body, err := io.ReadAll(resp.Body)
 	if err != nil {
-		return vol, nil, err
+		return volume.Volume{}, nil, err
 	}
+
+	var vol volume.Volume
 	rdr := bytes.NewReader(body)
 	err = json.NewDecoder(rdr).Decode(&vol)
 	return vol, body, err
