@@ -13,18 +13,16 @@ import (
 
 	"github.com/containerd/log"
 	"github.com/distribution/reference"
-	"github.com/docker/docker/api/types"
 	"github.com/docker/docker/api/types/backend"
 	containertypes "github.com/docker/docker/api/types/container"
 	"github.com/docker/docker/api/types/events"
 	"github.com/docker/docker/api/types/network"
 	"github.com/docker/docker/api/types/registry"
-	containerpkg "github.com/docker/docker/container"
 	"github.com/docker/docker/daemon"
 	"github.com/docker/docker/daemon/cluster/convert"
 	executorpkg "github.com/docker/docker/daemon/cluster/executor"
+	networkSettings "github.com/docker/docker/daemon/network"
 	"github.com/docker/docker/libnetwork"
-	"github.com/docker/docker/runconfig"
 	volumeopts "github.com/docker/docker/volume/service/opts"
 	gogotypes "github.com/gogo/protobuf/types"
 	"github.com/moby/swarmkit/v2/agent/exec"
@@ -181,13 +179,13 @@ func (c *containerAdapter) waitNodeAttachments(ctx context.Context) error {
 		// set a flag ready to true. if we try to get a network IP that doesn't
 		// exist yet, we will set this flag to "false"
 		ready := true
-		for _, attachment := range c.container.networksAttachments {
+		for _, nw := range c.container.networks {
 			// we only need node attachments (IP address) for overlay networks
 			// TODO(dperny): unsure if this will work with other network
 			// drivers, but i also don't think other network drivers use the
 			// node attachment IP address.
-			if attachment.Network.DriverState.Name == "overlay" {
-				if _, exists := attachmentStore.GetIPForNetwork(attachment.Network.ID); !exists {
+			if nw.DriverState.Name == "overlay" {
+				if _, exists := attachmentStore.GetIPForNetwork(nw.ID); !exists {
 					ready = false
 				}
 			}
@@ -208,12 +206,8 @@ func (c *containerAdapter) waitNodeAttachments(ctx context.Context) error {
 }
 
 func (c *containerAdapter) createNetworks(ctx context.Context) error {
-	for name := range c.container.networksAttachments {
-		ncr, err := c.container.networkCreateRequest(name)
-		if err != nil {
-			return err
-		}
-
+	for name, nw := range c.container.networks {
+		ncr := networkCreateRequest(name, nw)
 		if err := c.backend.CreateManagedNetwork(ncr); err != nil { // todo name missing
 			if _, ok := err.(libnetwork.NetworkNameError); ok {
 				continue
@@ -236,8 +230,8 @@ func (c *containerAdapter) removeNetworks(ctx context.Context) error {
 		errNoSuchNetwork     libnetwork.ErrNoSuchNetwork
 	)
 
-	for name, v := range c.container.networksAttachments {
-		if err := c.backend.DeleteManagedNetwork(v.Network.ID); err != nil {
+	for name, nw := range c.container.networks {
+		if err := c.backend.DeleteManagedNetwork(nw.ID); err != nil {
 			switch {
 			case errors.As(err, &activeEndpointsError):
 				continue
@@ -305,10 +299,10 @@ func (c *containerAdapter) create(ctx context.Context) error {
 	// need to make this normalization happen once we're sure we won't make a
 	// cross-OS API call.
 	if hostConfig.NetworkMode == "" || hostConfig.NetworkMode.IsDefault() {
-		hostConfig.NetworkMode = runconfig.DefaultDaemonNetworkMode()
+		hostConfig.NetworkMode = networkSettings.DefaultNetwork
 		if v, ok := netConfig.EndpointsConfig[network.NetworkDefault]; ok {
 			delete(netConfig.EndpointsConfig, network.NetworkDefault)
-			netConfig.EndpointsConfig[runconfig.DefaultDaemonNetworkMode().NetworkName()] = v
+			netConfig.EndpointsConfig[networkSettings.DefaultNetwork] = v
 		}
 	}
 
@@ -324,8 +318,8 @@ func (c *containerAdapter) create(ctx context.Context) error {
 		return err
 	}
 
-	container := c.container.task.Spec.GetContainer()
-	if container == nil {
+	ctr := c.container.task.Spec.GetContainer()
+	if ctr == nil {
 		return errors.New("unable to get container from task spec")
 	}
 
@@ -334,12 +328,12 @@ func (c *containerAdapter) create(ctx context.Context) error {
 	}
 
 	// configure secrets
-	secretRefs := convert.SecretReferencesFromGRPC(container.Secrets)
+	secretRefs := convert.SecretReferencesFromGRPC(ctr.Secrets)
 	if err := c.backend.SetContainerSecretReferences(cr.ID, secretRefs); err != nil {
 		return err
 	}
 
-	configRefs := convert.ConfigReferencesFromGRPC(container.Configs)
+	configRefs := convert.ConfigReferencesFromGRPC(ctr.Configs)
 	if err := c.backend.SetContainerConfigReferences(cr.ID, configRefs); err != nil {
 		return err
 	}
@@ -358,6 +352,8 @@ func (c *containerAdapter) checkMounts() error {
 			if _, err := os.Stat(mount.Source); os.IsNotExist(err) {
 				return fmt.Errorf("invalid bind mount source, source path not found: %s", mount.Source)
 			}
+		default:
+			// TODO(thaJeztah): make switch exhaustive; add api.MountTypeVolume, api.MountTypeTmpfs, api.MountTypeNamedPipe, api.MountTypeCluster
 		}
 	}
 
@@ -372,13 +368,13 @@ func (c *containerAdapter) start(ctx context.Context) error {
 	return c.backend.ContainerStart(ctx, c.container.name(), "", "")
 }
 
-func (c *containerAdapter) inspect(ctx context.Context) (types.ContainerJSON, error) {
-	cs, err := c.backend.ContainerInspectCurrent(ctx, c.container.name(), false)
+func (c *containerAdapter) inspect(ctx context.Context) (containertypes.InspectResponse, error) {
+	cs, err := c.backend.ContainerInspect(ctx, c.container.name(), backend.ContainerInspectOptions{})
 	if ctx.Err() != nil {
-		return types.ContainerJSON{}, ctx.Err()
+		return containertypes.InspectResponse{}, ctx.Err()
 	}
 	if err != nil {
-		return types.ContainerJSON{}, err
+		return containertypes.InspectResponse{}, err
 	}
 	return *cs, nil
 }
@@ -419,8 +415,8 @@ func (c *containerAdapter) events(ctx context.Context) <-chan events.Message {
 	return eventsq
 }
 
-func (c *containerAdapter) wait(ctx context.Context) (<-chan containerpkg.StateStatus, error) {
-	return c.backend.ContainerWait(ctx, c.container.nameOrID(), containerpkg.WaitConditionNotRunning)
+func (c *containerAdapter) wait(ctx context.Context) (<-chan containertypes.StateStatus, error) {
+	return c.backend.ContainerWait(ctx, c.container.nameOrID(), containertypes.WaitConditionNotRunning)
 }
 
 func (c *containerAdapter) shutdown(ctx context.Context) error {
@@ -447,7 +443,6 @@ func (c *containerAdapter) remove(ctx context.Context) error {
 func (c *containerAdapter) createVolumes(ctx context.Context) error {
 	// Create plugin volumes that are embedded inside a Mount
 	for _, mount := range c.container.task.Spec.GetContainer().Mounts {
-		mount := mount
 		if mount.Type != api.MountTypeVolume {
 			continue
 		}
@@ -548,6 +543,8 @@ func (c *containerAdapter) logs(ctx context.Context, options api.LogSubscription
 				apiOptions.ShowStdout = true
 			case api.LogStreamStderr:
 				apiOptions.ShowStderr = true
+			default:
+				// TODO(thaJeztah): make switch exhaustive; add api.LogStreamUnknown
 			}
 		}
 	}
@@ -557,16 +554,3 @@ func (c *containerAdapter) logs(ctx context.Context, options api.LogSubscription
 	}
 	return msgs, nil
 }
-
-// todo: typed/wrapped errors
-func isContainerCreateNameConflict(err error) bool {
-	return strings.Contains(err.Error(), "Conflict. The name")
-}
-
-func isUnknownContainer(err error) bool {
-	return strings.Contains(err.Error(), "No such container:")
-}
-
-func isStoppedContainer(err error) bool {
-	return strings.Contains(err.Error(), "is already stopped")
-}
