@@ -10,7 +10,7 @@ import (
 	"strings"
 
 	"github.com/Microsoft/hcsshim"
-	coci "github.com/containerd/containerd/oci"
+	coci "github.com/containerd/containerd/v2/pkg/oci"
 	"github.com/containerd/log"
 	"github.com/docker/docker/api/types/backend"
 	containertypes "github.com/docker/docker/api/types/container"
@@ -20,7 +20,7 @@ import (
 	"github.com/docker/docker/image"
 	"github.com/docker/docker/oci"
 	"github.com/docker/docker/pkg/system"
-	specs "github.com/opencontainers/runtime-spec/specs-go"
+	"github.com/opencontainers/runtime-spec/specs-go"
 	"github.com/pkg/errors"
 	"golang.org/x/sys/windows/registry"
 )
@@ -478,7 +478,7 @@ func (daemon *Daemon) mergeUlimits(c *containertypes.HostConfig, daemonCfg *conf
 // listing only the methods we care about here.
 // It's mainly useful to easily allow mocking the registry in tests.
 type registryKey interface {
-	GetStringValue(name string) (val string, valtype uint32, err error)
+	GetStringValue(name string) (val string, valType uint32, err error)
 	Close() error
 }
 
@@ -526,7 +526,8 @@ func readCredentialSpecFile(id, root, location string) (string, error) {
 	return string(bcontents[:]), nil
 }
 
-func setupWindowsDevices(devices []containertypes.DeviceMapping) (specDevices []specs.WindowsDevice, err error) {
+func setupWindowsDevices(devices []containertypes.DeviceMapping) ([]specs.WindowsDevice, error) {
+	var specDevices []specs.WindowsDevice
 	for _, deviceMapping := range devices {
 		if strings.HasPrefix(deviceMapping.PathOnHost, "class/") {
 			specDevices = append(specDevices, specs.WindowsDevice{
@@ -550,3 +551,8 @@ func setupWindowsDevices(devices []containertypes.DeviceMapping) (specDevices []
 
 	return specDevices, nil
 }
+
+// getUser is a no-op on Windows.
+func getUser(c *container.Container, username string) (specs.User, error) {
+	return specs.User{}, nil
+}
