@@ -13,7 +13,6 @@ import (
 
 	"github.com/Microsoft/hcsshim/osversion"
 	"github.com/containerd/log"
-	"github.com/containerd/platforms"
 	"github.com/docker/distribution"
 	"github.com/docker/distribution/manifest/manifestlist"
 	"github.com/docker/distribution/manifest/schema2"
@@ -151,10 +150,3 @@ func checkImageCompatibility(imageOS, imageOSVersion string) error {
 	}
 	return nil
 }
-
-func formatPlatform(platform ocispec.Platform) string {
-	if platform.OS == "" {
-		platform = platforms.DefaultSpec()
-	}
-	return fmt.Sprintf("%s %s", platforms.Format(platform), osversion.Get().ToString())
-}
