@@ -4,8 +4,6 @@ import (
 	"context"
 	"fmt"
 	"io"
-	"os"
-	"runtime"
 	"sort"
 	"strings"
 	"sync"
@@ -13,7 +11,6 @@ import (
 	"github.com/containerd/log"
 	"github.com/distribution/reference"
 	"github.com/docker/distribution"
-	"github.com/docker/distribution/manifest/schema1"
 	"github.com/docker/distribution/manifest/schema2"
 	"github.com/docker/distribution/registry/api/errcode"
 	"github.com/docker/distribution/registry/client"
@@ -25,7 +22,6 @@ import (
 	"github.com/docker/docker/pkg/progress"
 	"github.com/docker/docker/pkg/stringid"
 	"github.com/docker/docker/registry"
-	"github.com/docker/libtrust"
 	"github.com/opencontainers/go-digest"
 	"github.com/pkg/errors"
 )
@@ -38,12 +34,12 @@ const (
 // newPusher creates a new pusher for pushing to a v2 registry.
 // The parameters are passed through to the underlying pusher implementation for
 // use during the actual push operation.
-func newPusher(ref reference.Named, endpoint registry.APIEndpoint, repoInfo *registry.RepositoryInfo, config *ImagePushConfig) *pusher {
+func newPusher(ref reference.Named, endpoint registry.APIEndpoint, repoName reference.Named, config *ImagePushConfig) *pusher {
 	return &pusher{
 		metadataService: metadata.NewV2MetadataService(config.MetadataStore),
 		ref:             ref,
 		endpoint:        endpoint,
-		repoInfo:        repoInfo,
+		repoName:        repoName,
 		config:          config,
 	}
 }
@@ -52,7 +48,7 @@ type pusher struct {
 	metadataService metadata.V2MetadataService
 	ref             reference.Named
 	endpoint        registry.APIEndpoint
-	repoInfo        *registry.RepositoryInfo
+	repoName        reference.Named
 	config          *ImagePushConfig
 	repo            distribution.Repository
 
@@ -74,7 +70,7 @@ type pushState struct {
 func (p *pusher) push(ctx context.Context) (err error) {
 	p.pushState.remoteLayers = make(map[layer.DiffID]distribution.Descriptor)
 
-	p.repo, err = newRepository(ctx, p.repoInfo, p.endpoint, p.config.MetaHeaders, p.config.AuthConfig, "push", "pull")
+	p.repo, err = newRepository(ctx, p.repoName, p.endpoint, p.config.MetaHeaders, p.config.AuthConfig, "push", "pull")
 	p.pushState.hasAuthInfo = p.config.AuthConfig.RegistryToken != "" || (p.config.AuthConfig.Username != "" && p.config.AuthConfig.Password != "")
 	if err != nil {
 		log.G(ctx).Debugf("Error getting v2 registry: %v", err)
@@ -82,7 +78,8 @@ func (p *pusher) push(ctx context.Context) (err error) {
 	}
 
 	if err = p.pushRepository(ctx); err != nil {
-		if continueOnError(err, p.endpoint.Mirror) {
+		// [Service.LookupPushEndpoints] never returns mirror endpoint.
+		if continueOnError(err, false) {
 			return fallbackError{
 				err:         err,
 				transportOK: true,
@@ -118,7 +115,7 @@ func (p *pusher) pushRepository(ctx context.Context) (err error) {
 	}
 
 	if pushed == 0 {
-		return fmt.Errorf("no tags to push for %s", reference.FamiliarName(p.repoInfo.Name))
+		return fmt.Errorf("no tags to push for %s", reference.FamiliarName(p.repoName))
 	}
 
 	return nil
@@ -153,9 +150,8 @@ func (p *pusher) pushTag(ctx context.Context, ref reference.NamedTagged, id dige
 	descriptorTemplate := pushDescriptor{
 		metadataService: p.metadataService,
 		hmacKey:         hmacKey,
-		repoInfo:        p.repoInfo.Name,
+		repoName:        p.repoName,
 		ref:             p.ref,
-		endpoint:        p.endpoint,
 		repo:            p.repo,
 		pushState:       &p.pushState,
 	}
@@ -188,60 +184,24 @@ func (p *pusher) pushTag(ctx context.Context, ref reference.NamedTagged, id dige
 
 	putOptions := []distribution.ManifestServiceOption{distribution.WithTag(ref.Tag())}
 	if _, err = manSvc.Put(ctx, manifest, putOptions...); err != nil {
-		if runtime.GOOS == "windows" {
-			log.G(ctx).Warnf("failed to upload schema2 manifest: %v", err)
-			return err
+		if err.Error() == "tag invalid" {
+			msg := "[DEPRECATED] support for pushing manifest v2 schema1 images has been removed. More information at https://docs.docker.com/registry/spec/deprecated-schema-v1/"
+			log.G(ctx).WithError(err).Error(msg)
+			err = errors.Wrap(err, msg)
 		}
-
-		// This is a temporary environment variables used in CI to allow pushing
-		// manifest v2 schema 1 images to test-registries used for testing *pulling*
-		// these images.
-		if os.Getenv("DOCKER_ALLOW_SCHEMA1_PUSH_DONOTUSE") == "" {
-			if err.Error() == "tag invalid" {
-				msg := "[DEPRECATED] support for pushing manifest v2 schema1 images has been removed. More information at https://docs.docker.com/registry/spec/deprecated-schema-v1/"
-				log.G(ctx).WithError(err).Error(msg)
-				return errors.Wrap(err, msg)
-			}
-			return err
-		}
-
-		log.G(ctx).Warnf("failed to upload schema2 manifest: %v - falling back to schema1", err)
-
-		// Note: this fallback is deprecated, see log messages below
-		manifestRef, err := reference.WithTag(p.repo.Named(), ref.Tag())
-		if err != nil {
-			return err
-		}
-		pk, err := libtrust.GenerateECP256PrivateKey()
-		if err != nil {
-			return errors.Wrap(err, "unexpected error generating private key")
-		}
-		builder = schema1.NewConfigManifestBuilder(p.repo.Blobs(ctx), pk, manifestRef, imgConfig)
-		manifest, err = manifestFromBuilder(ctx, builder, descriptors)
-		if err != nil {
-			return err
-		}
-
-		if _, err = manSvc.Put(ctx, manifest, putOptions...); err != nil {
-			return err
-		}
-
-		// schema2 failed but schema1 succeeded
-		msg := fmt.Sprintf("[DEPRECATION NOTICE] support for pushing manifest v2 schema1 images will be removed in an upcoming release. Please contact admins of the %s registry NOW to avoid future disruption. More information at https://docs.docker.com/registry/spec/deprecated-schema-v1/", reference.Domain(ref))
-		log.G(ctx).Warn(msg)
-		progress.Message(p.config.ProgressOutput, "", msg)
+		return err
 	}
 
 	var canonicalManifest []byte
 
 	switch v := manifest.(type) {
-	case *schema1.SignedManifest:
-		canonicalManifest = v.Canonical
 	case *schema2.DeserializedManifest:
 		_, canonicalManifest, err = v.Payload()
 		if err != nil {
 			return err
 		}
+	default:
+		return fmt.Errorf("unknown manifest type %T", v)
 	}
 
 	manifestDigest := digest.FromBytes(canonicalManifest)
@@ -274,9 +234,8 @@ type pushDescriptor struct {
 	layer            PushLayer
 	metadataService  metadata.V2MetadataService
 	hmacKey          []byte
-	repoInfo         reference.Named
+	repoName         reference.Named
 	ref              reference.Named
-	endpoint         registry.APIEndpoint
 	repo             distribution.Repository
 	pushState        *pushState
 	remoteDescriptor distribution.Descriptor
@@ -297,16 +256,6 @@ func (pd *pushDescriptor) DiffID() layer.DiffID {
 }
 
 func (pd *pushDescriptor) Upload(ctx context.Context, progressOutput progress.Output) (distribution.Descriptor, error) {
-	// Skip foreign layers unless this registry allows nondistributable artifacts.
-	if !pd.endpoint.AllowNondistributableArtifacts {
-		if fs, ok := pd.layer.(distribution.Describable); ok {
-			if d := fs.Descriptor(); len(d.URLs) > 0 {
-				progress.Update(progressOutput, pd.ID(), "Skipped foreign layer")
-				return d, nil
-			}
-		}
-	}
-
 	diffID := pd.DiffID()
 
 	pd.pushState.Lock()
@@ -338,7 +287,7 @@ func (pd *pushDescriptor) Upload(ctx context.Context, progressOutput progress.Ou
 	var layerUpload distribution.BlobWriter
 
 	// Attempt to find another repository in the same registry to mount the layer from to avoid an unnecessary upload
-	candidates := getRepositoryMountCandidates(pd.repoInfo, pd.hmacKey, maxMountAttempts, metaData)
+	candidates := getRepositoryMountCandidates(pd.repoName, pd.hmacKey, maxMountAttempts, metaData)
 	isUnauthorizedError := false
 	for _, mc := range candidates {
 		mountCandidate := mc
@@ -387,7 +336,7 @@ func (pd *pushDescriptor) Upload(ctx context.Context, progressOutput progress.Ou
 			// Cache mapping from this layer's DiffID to the blobsum
 			if err := pd.metadataService.TagAndAdd(diffID, pd.hmacKey, metadata.V2Metadata{
 				Digest:           err.Descriptor.Digest,
-				SourceRepository: pd.repoInfo.Name(),
+				SourceRepository: pd.repoName.Name(),
 			}); err != nil {
 				return distribution.Descriptor{}, xfer.DoNotRetry{Err: err}
 			}
@@ -411,7 +360,7 @@ func (pd *pushDescriptor) Upload(ctx context.Context, progressOutput progress.Ou
 		// when error is unauthorizedError and user don't hasAuthInfo that's the case user don't has right to push layer to register
 		// and he hasn't login either, in this case candidate cache should be removed
 		if len(mountCandidate.SourceRepository) > 0 &&
-			!(isUnauthorizedError && !pd.pushState.hasAuthInfo) &&
+			(!isUnauthorizedError || pd.pushState.hasAuthInfo) &&
 			(metadata.CheckV2MetadataHMAC(&mountCandidate, pd.hmacKey) ||
 				len(mountCandidate.HMAC) == 0) {
 			cause := "blob mount failure"
@@ -506,7 +455,7 @@ func (pd *pushDescriptor) uploadUsingSession(
 	// Cache mapping from this layer's DiffID to the blobsum
 	if err := pd.metadataService.TagAndAdd(diffID, pd.hmacKey, metadata.V2Metadata{
 		Digest:           pushDigest,
-		SourceRepository: pd.repoInfo.Name(),
+		SourceRepository: pd.repoName.Name(),
 	}); err != nil {
 		return distribution.Descriptor{}, xfer.DoNotRetry{Err: err}
 	}
@@ -535,17 +484,17 @@ func (pd *pushDescriptor) layerAlreadyExists(
 	checkOtherRepositories bool,
 	maxExistenceCheckAttempts int,
 	v2Metadata []metadata.V2Metadata,
-) (desc distribution.Descriptor, exists bool, err error) {
+) (_ distribution.Descriptor, exists bool, _ error) {
 	// filter the metadata
 	candidates := []metadata.V2Metadata{}
 	for _, meta := range v2Metadata {
-		if len(meta.SourceRepository) > 0 && !checkOtherRepositories && meta.SourceRepository != pd.repoInfo.Name() {
+		if len(meta.SourceRepository) > 0 && !checkOtherRepositories && meta.SourceRepository != pd.repoName.Name() {
 			continue
 		}
 		candidates = append(candidates, meta)
 	}
 	// sort the candidates by similarity
-	sortV2MetadataByLikenessAndAge(pd.repoInfo, pd.hmacKey, candidates)
+	sortV2MetadataByLikenessAndAge(pd.repoName, pd.hmacKey, candidates)
 
 	digestToMetadata := make(map[digest.Digest]*metadata.V2Metadata)
 	// an array of unique blob digests ordered from the best mount candidates to worst
@@ -555,11 +504,11 @@ func (pd *pushDescriptor) layerAlreadyExists(
 			break
 		}
 		meta := &candidates[i]
-		if _, exists := digestToMetadata[meta.Digest]; exists {
+		if _, ok := digestToMetadata[meta.Digest]; ok {
 			// keep reference just to the first mapping (the best mount candidate)
 			continue
 		}
-		if _, exists := pd.checkedDigests[meta.Digest]; exists {
+		if _, ok := pd.checkedDigests[meta.Digest]; ok {
 			// existence of this digest has already been tested
 			continue
 		}
@@ -567,19 +516,22 @@ func (pd *pushDescriptor) layerAlreadyExists(
 		layerDigests = append(layerDigests, meta.Digest)
 	}
 
+	var desc distribution.Descriptor
+
 attempts:
 	for _, dgst := range layerDigests {
 		meta := digestToMetadata[dgst]
-		log.G(ctx).Debugf("Checking for presence of layer %s (%s) in %s", diffID, dgst, pd.repoInfo.Name())
+		log.G(ctx).Debugf("Checking for presence of layer %s (%s) in %s", diffID, dgst, pd.repoName.Name())
+		var err error
 		desc, err = pd.repo.Blobs(ctx).Stat(ctx, dgst)
 		pd.checkedDigests[meta.Digest] = struct{}{}
 		switch err {
 		case nil:
-			if m, ok := digestToMetadata[desc.Digest]; !ok || m.SourceRepository != pd.repoInfo.Name() || !metadata.CheckV2MetadataHMAC(m, pd.hmacKey) {
+			if m, ok := digestToMetadata[desc.Digest]; !ok || m.SourceRepository != pd.repoName.Name() || !metadata.CheckV2MetadataHMAC(m, pd.hmacKey) {
 				// cache mapping from this layer's DiffID to the blobsum
 				if err := pd.metadataService.TagAndAdd(diffID, pd.hmacKey, metadata.V2Metadata{
 					Digest:           desc.Digest,
-					SourceRepository: pd.repoInfo.Name(),
+					SourceRepository: pd.repoName.Name(),
 				}); err != nil {
 					return distribution.Descriptor{}, false, xfer.DoNotRetry{Err: err}
 				}
@@ -588,12 +540,14 @@ attempts:
 			exists = true
 			break attempts
 		case distribution.ErrBlobUnknown:
-			if meta.SourceRepository == pd.repoInfo.Name() {
+			if meta.SourceRepository == pd.repoName.Name() {
 				// remove the mapping to the target repository
-				pd.metadataService.Remove(*meta)
+				if err := pd.metadataService.Remove(*meta); err != nil {
+					log.G(ctx).WithError(err).Debug("Failed remove metadata")
+				}
 			}
 		default:
-			log.G(ctx).WithError(err).Debugf("Failed to check for presence of layer %s (%s) in %s", diffID, dgst, pd.repoInfo.Name())
+			log.G(ctx).WithError(err).Debugf("Failed to check for presence of layer %s (%s) in %s", diffID, dgst, pd.repoName.Name())
 		}
 	}
 
