@@ -1,5 +1,4 @@
 //go:build !windows
-// +build !windows
 
 package libnetwork
 
@@ -12,12 +11,6 @@ func (ep *Endpoint) DriverInfo() (map[string]interface{}, error) {
 		return nil, err
 	}
 
-	if sb, ok := ep.getSandbox(); ok {
-		if gwep := sb.getEndpointInGWNetwork(); gwep != nil && gwep.ID() != ep.ID() {
-			return gwep.DriverInfo()
-		}
-	}
-
 	n, err := ep.getNetworkFromStore()
 	if err != nil {
 		return nil, fmt.Errorf("could not find network in store for driver info: %v", err)
