@@ -1,3 +1,6 @@
+// FIXME(thaJeztah): remove once we are a module; the go:build directive prevents go from downgrading language version to go1.16:
+//go:build go1.23
+
 package networkdb
 
 //go:generate protoc -I=. -I=../../vendor/ --gogofaster_out=import_path=github.com/docker/docker/libnetwork/networkdb:. networkdb.proto
@@ -15,7 +18,7 @@ import (
 	"github.com/docker/docker/libnetwork/types"
 	"github.com/docker/docker/pkg/stringid"
 	"github.com/docker/go-events"
-	iradix "github.com/hashicorp/go-immutable-radix"
+	iradix "github.com/hashicorp/go-immutable-radix/v2"
 	"github.com/hashicorp/memberlist"
 	"github.com/hashicorp/serf/serf"
 )
@@ -44,7 +47,7 @@ type NetworkDB struct {
 
 	// All the tree index (byTable, byNetwork) that we maintain
 	// the db.
-	indexes map[int]*iradix.Tree
+	indexes map[int]*iradix.Tree[*entry]
 
 	// Memberlist we use to drive the cluster.
 	memberlist *memberlist.Memberlist
@@ -189,7 +192,7 @@ type Config struct {
 	// reapEntryInterval duration of a deleted entry before being garbage collected
 	reapEntryInterval time.Duration
 
-	// reapNetworkInterval duration of a delted network before being garbage collected
+	// reapNetworkInterval duration of a deleted network before being garbage collected
 	// NOTE this MUST always be higher than reapEntryInterval
 	reapNetworkInterval time.Duration
 
@@ -249,14 +252,27 @@ func DefaultConfig() *Config {
 // New creates a new instance of NetworkDB using the Config passed by
 // the caller.
 func New(c *Config) (*NetworkDB, error) {
+	nDB := new(c)
+	log.G(context.TODO()).Infof("New memberlist node - Node:%v will use memberlist nodeID:%v with config:%+v", c.Hostname, c.NodeID, c)
+	if err := nDB.clusterInit(); err != nil {
+		return nil, err
+	}
+
+	return nDB, nil
+}
+
+func new(c *Config) *NetworkDB {
 	// The garbage collection logic for entries leverage the presence of the network.
 	// For this reason the expiration time of the network is put slightly higher than the entry expiration so that
 	// there is at least 5 extra cycle to make sure that all the entries are properly deleted before deleting the network.
 	c.reapNetworkInterval = c.reapEntryInterval + 5*reapPeriod
 
-	nDB := &NetworkDB{
-		config:         c,
-		indexes:        make(map[int]*iradix.Tree),
+	return &NetworkDB{
+		config: c,
+		indexes: map[int]*iradix.Tree[*entry]{
+			byTable:   iradix.New[*entry](),
+			byNetwork: iradix.New[*entry](),
+		},
 		networks:       make(map[string]map[string]*network),
 		nodes:          make(map[string]*node),
 		failedNodes:    make(map[string]*node),
@@ -265,16 +281,6 @@ func New(c *Config) (*NetworkDB, error) {
 		bulkSyncAckTbl: make(map[string]chan struct{}),
 		broadcaster:    events.NewBroadcaster(),
 	}
-
-	nDB.indexes[byTable] = iradix.New()
-	nDB.indexes[byNetwork] = iradix.New()
-
-	log.G(context.TODO()).Infof("New memberlist node - Node:%v will use memberlist nodeID:%v with config:%+v", c.Hostname, c.NodeID, c)
-	if err := nDB.clusterInit(); err != nil {
-		return nil, err
-	}
-
-	return nDB, nil
 }
 
 // Join joins this NetworkDB instance with a list of peer NetworkDB
@@ -337,15 +343,19 @@ func (nDB *NetworkDB) Peers(nid string) []PeerInfo {
 func (nDB *NetworkDB) GetEntry(tname, nid, key string) ([]byte, error) {
 	nDB.RLock()
 	defer nDB.RUnlock()
-	entry, err := nDB.getEntry(tname, nid, key)
+	v, err := nDB.getEntry(tname, nid, key)
 	if err != nil {
 		return nil, err
 	}
-	if entry != nil && entry.deleting {
+	if v != nil && v.deleting {
 		return nil, types.NotFoundErrorf("entry in table %s network id %s and key %s deleted and pending garbage collection", tname, nid, key)
 	}
 
-	return entry.value, nil
+	// note: this panics if a nil entry was stored in the table; after
+	// discussion, we decided to not gracefully handle this situation as
+	// this would be an unexpected situation;
+	// see https://github.com/moby/moby/pull/48157#discussion_r1674428635
+	return v.value, nil
 }
 
 func (nDB *NetworkDB) getEntry(tname, nid, key string) (*entry, error) {
@@ -354,7 +364,7 @@ func (nDB *NetworkDB) getEntry(tname, nid, key string) (*entry, error) {
 		return nil, types.NotFoundErrorf("could not get entry in table %s with network id %s and key %s", tname, nid, key)
 	}
 
-	return e.(*entry), nil
+	return e, nil
 }
 
 // CreateEntry creates a table entry in NetworkDB for given (network,
@@ -422,15 +432,17 @@ type TableElem struct {
 // GetTableByNetwork walks the networkdb by the give table and network id and
 // returns a map of keys and values
 func (nDB *NetworkDB) GetTableByNetwork(tname, nid string) map[string]*TableElem {
+	nDB.RLock()
+	root := nDB.indexes[byTable].Root()
+	nDB.RUnlock()
 	entries := make(map[string]*TableElem)
-	nDB.indexes[byTable].Root().WalkPrefix([]byte(fmt.Sprintf("/%s/%s", tname, nid)), func(k []byte, v interface{}) bool {
-		entry := v.(*entry)
-		if entry.deleting {
+	root.WalkPrefix([]byte(fmt.Sprintf("/%s/%s", tname, nid)), func(k []byte, v *entry) bool {
+		if v.deleting {
 			return false
 		}
 		key := string(k)
 		key = key[strings.LastIndex(key, "/")+1:]
-		entries[key] = &TableElem{Value: entry.value, owner: entry.node}
+		entries[key] = &TableElem{Value: v.value, owner: v.node}
 		return false
 	})
 	return entries
@@ -502,12 +514,8 @@ func (nDB *NetworkDB) deleteNodeNetworkEntries(nid, node string) {
 	isNodeLocal := node == nDB.config.NodeID
 
 	nDB.indexes[byNetwork].Root().WalkPrefix([]byte("/"+nid),
-		func(path []byte, v interface{}) bool {
-			oldEntry := v.(*entry)
-			params := strings.Split(string(path[1:]), "/")
-			nid := params[0]
-			tname := params[1]
-			key := params[2]
+		func(path []byte, v *entry) bool {
+			oldEntry := v
 
 			// If the entry is owned by a remote node and this node is not leaving the network
 			if oldEntry.node != node && !isNodeLocal {
@@ -521,7 +529,7 @@ func (nDB *NetworkDB) deleteNodeNetworkEntries(nid, node string) {
 				return false
 			}
 
-			entry := &entry{
+			newEntry := &entry{
 				ltime:    oldEntry.ltime,
 				node:     oldEntry.node,
 				value:    oldEntry.value,
@@ -532,32 +540,34 @@ func (nDB *NetworkDB) deleteNodeNetworkEntries(nid, node string) {
 			// we arrived at this point in 2 cases:
 			// 1) this entry is owned by the node that is leaving the network
 			// 2) the local node is leaving the network
+			params := strings.Split(string(path[1:]), "/")
+			nwID, tName, key := params[0], params[1], params[2]
 			if oldEntry.node == node {
 				if isNodeLocal {
 					// TODO fcrisciani: this can be removed if there is no way to leave the network
 					// without doing a delete of all the objects
-					entry.ltime++
+					newEntry.ltime++
 				}
 
 				if !oldEntry.deleting {
-					nDB.createOrUpdateEntry(nid, tname, key, entry)
+					nDB.createOrUpdateEntry(nwID, tName, key, newEntry)
 				}
 			} else {
 				// the local node is leaving the network, all the entries of remote nodes can be safely removed
-				nDB.deleteEntry(nid, tname, key)
+				nDB.deleteEntry(nwID, tName, key)
 			}
 
 			// Notify to the upper layer only entries not already marked for deletion
 			if !oldEntry.deleting {
-				nDB.broadcaster.Write(makeEvent(opDelete, tname, nid, key, entry.value))
+				nDB.broadcaster.Write(makeEvent(opDelete, tName, nwID, key, newEntry.value))
 			}
 			return false
 		})
 }
 
 func (nDB *NetworkDB) deleteNodeTableEntries(node string) {
-	nDB.indexes[byTable].Root().Walk(func(path []byte, v interface{}) bool {
-		oldEntry := v.(*entry)
+	nDB.indexes[byTable].Root().Walk(func(path []byte, v *entry) bool {
+		oldEntry := v
 		if oldEntry.node != node {
 			return false
 		}
@@ -581,21 +591,14 @@ func (nDB *NetworkDB) deleteNodeTableEntries(node string) {
 // value. The walk stops if the passed function returns a true.
 func (nDB *NetworkDB) WalkTable(tname string, fn func(string, string, []byte, bool) bool) error {
 	nDB.RLock()
-	values := make(map[string]interface{})
-	nDB.indexes[byTable].Root().WalkPrefix([]byte("/"+tname), func(path []byte, v interface{}) bool {
-		values[string(path)] = v
-		return false
-	})
+	root := nDB.indexes[byTable].Root()
 	nDB.RUnlock()
-
-	for k, v := range values {
-		params := strings.Split(k[1:], "/")
+	root.WalkPrefix([]byte("/"+tname), func(path []byte, v *entry) bool {
+		params := strings.Split(string(path[1:]), "/")
 		nid := params[1]
 		key := params[2]
-		if fn(nid, key, v.(*entry).value, v.(*entry).deleting) {
-			return nil
-		}
-	}
+		return fn(nid, key, v.value, v.deleting)
+	})
 
 	return nil
 }
@@ -724,7 +727,7 @@ func (nDB *NetworkDB) deleteNetworkNode(nid string, nodeName string) {
 	nDB.networkNodes[nid] = newNodes
 }
 
-// findCommonnetworks find the networks that both this node and the
+// findCommonNetworks find the networks that both this node and the
 // passed node have joined.
 func (nDB *NetworkDB) findCommonNetworks(nodeName string) []string {
 	nDB.RLock()
@@ -754,9 +757,9 @@ func (nDB *NetworkDB) updateLocalNetworkTime() {
 
 // createOrUpdateEntry this function handles the creation or update of entries into the local
 // tree store. It is also used to keep in sync the entries number of the network (all tables are aggregated)
-func (nDB *NetworkDB) createOrUpdateEntry(nid, tname, key string, entry interface{}) (okTable bool, okNetwork bool) {
-	nDB.indexes[byTable], _, okTable = nDB.indexes[byTable].Insert([]byte(fmt.Sprintf("/%s/%s/%s", tname, nid, key)), entry)
-	nDB.indexes[byNetwork], _, okNetwork = nDB.indexes[byNetwork].Insert([]byte(fmt.Sprintf("/%s/%s/%s", nid, tname, key)), entry)
+func (nDB *NetworkDB) createOrUpdateEntry(nid, tname, key string, v *entry) (okTable bool, okNetwork bool) {
+	nDB.indexes[byTable], _, okTable = nDB.indexes[byTable].Insert([]byte(fmt.Sprintf("/%s/%s/%s", tname, nid, key)), v)
+	nDB.indexes[byNetwork], _, okNetwork = nDB.indexes[byNetwork].Insert([]byte(fmt.Sprintf("/%s/%s/%s", nid, tname, key)), v)
 	if !okNetwork {
 		// Add only if it is an insert not an update
 		n, ok := nDB.networks[nDB.config.NodeID][nid]
