@@ -30,10 +30,10 @@ import (
 // backend resolver.
 type DNSBackend interface {
 	// ResolveName resolves a service name to an IPv4 or IPv6 address by searching
-	// the networks the sandbox is connected to. For IPv6 queries, second return
-	// value will be true if the name exists in docker domain but doesn't have an
-	// IPv6 address. Such queries shouldn't be forwarded to external nameservers.
-	ResolveName(ctx context.Context, name string, iplen int) ([]net.IP, bool)
+	// the networks the sandbox is connected to. The second return value will be
+	// true if the name exists in docker domain, even if there are no addresses of
+	// the required type. Such queries shouldn't be forwarded to external nameservers.
+	ResolveName(ctx context.Context, name string, ipType int) ([]net.IP, bool)
 	// ResolveIP returns the service name for the passed in IP. IP is in reverse dotted
 	// notation; the format used for DNS PTR records
 	ResolveIP(ctx context.Context, name string) string
@@ -91,7 +91,7 @@ type Resolver struct {
 	logger        *log.Entry
 
 	fwdSem      *semaphore.Weighted // Limit the number of concurrent external DNS requests in-flight
-	logInverval rate.Sometimes      // Rate-limit logging about hitting the fwdSem limit
+	logInterval rate.Sometimes      // Rate-limit logging about hitting the fwdSem limit
 }
 
 // NewResolver creates a new instance of the Resolver
@@ -101,7 +101,7 @@ func NewResolver(address string, proxyDNS bool, backend DNSBackend) *Resolver {
 		err:         fmt.Errorf("setup not done yet"),
 		startCh:     make(chan struct{}, 1),
 		fwdSem:      semaphore.NewWeighted(maxConcurrent),
-		logInverval: rate.Sometimes{Interval: logInterval},
+		logInterval: rate.Sometimes{Interval: logInterval},
 	}
 	r.listenAddress, _ = netip.ParseAddr(address)
 	r.proxyDNS.Store(proxyDNS)
@@ -180,8 +180,8 @@ func (r *Resolver) Start() error {
 		return r.err
 	}
 
-	if err := r.setupIPTable(); err != nil {
-		return fmt.Errorf("setting up IP table rules failed: %v", err)
+	if err := r.setupNAT(context.TODO()); err != nil {
+		return fmt.Errorf("setting up DNAT/SNAT rules failed: %v", err)
 	}
 
 	s := &dns.Server{Handler: dns.HandlerFunc(r.serveDNS), PacketConn: r.conn}
@@ -311,20 +311,9 @@ func (r *Resolver) handleMXQuery(ctx context.Context, query *dns.Msg) (*dns.Msg,
 }
 
 func (r *Resolver) handleIPQuery(ctx context.Context, query *dns.Msg, ipType int) (*dns.Msg, error) {
-	var (
-		addr     []net.IP
-		ipv6Miss bool
-		name     = query.Question[0].Name
-	)
-	addr, ipv6Miss = r.backend.ResolveName(ctx, name, ipType)
-
-	if addr == nil && ipv6Miss {
-		// Send a reply without any Answer sections
-		r.log(ctx).Debugf("[resolver] lookup name %s present without IPv6 address", name)
-		resp := createRespMsg(query)
-		return resp, nil
-	}
-	if addr == nil {
+	name := query.Question[0].Name
+	addr, ok := r.backend.ResolveName(ctx, name, ipType)
+	if !ok {
 		return nil, nil
 	}
 
@@ -559,7 +548,7 @@ func (r *Resolver) forwardExtDNS(ctx context.Context, proto string, remoteAddr n
 
 		if err != nil {
 			if errors.Is(err, context.DeadlineExceeded) {
-				r.logInverval.Do(func() {
+				r.logInterval.Do(func() {
 					r.log(ctx).Errorf("[resolver] more than %v concurrent queries", maxConcurrent)
 				})
 			}
@@ -600,9 +589,7 @@ func (r *Resolver) forwardExtDNS(ctx context.Context, proto string, remoteAddr n
 			r.log(ctx).Debugf("[resolver] external DNS %s:%s returned response with no answers:\n%s", proto, extDNS.IPStr, resp)
 		}
 		resp.Compress = true
-		span.AddEvent("response from upstream server", trace.WithAttributes(
-			attribute.String("libnet.resolver.resp", resp.String()),
-		))
+		span.AddEvent("response from upstream server")
 		return resp
 	}
 
@@ -647,7 +634,7 @@ func (r *Resolver) exchange(ctx context.Context, proto string, extDNS extDNSEntr
 		// truncating them on our end to forward verbatim to the client.
 		// Some DNS servers (e.g. Mikrotik RouterOS) don't support
 		// EDNS(0) and may send replies over UDP longer than 512 bytes
-		// regardless of what size limit, if any, was advertized in the
+		// regardless of what size limit, if any, was advertised in the
 		// query message. Note that ExchangeWithConn will override this
 		// value if it detects an EDNS OPT record in query so only
 		// oversized replies to non-EDNS queries will benefit.
