@@ -1,22 +1,28 @@
+// FIXME(thaJeztah): remove once we are a module; the go:build directive prevents go from downgrading language version to go1.16:
+//go:build go1.23
+
 package daemon // import "github.com/docker/docker/testutil/daemon"
 
 import (
 	"bufio"
 	"context"
 	"encoding/json"
+	"fmt"
 	"io"
 	"net/http"
 	"os"
 	"os/exec"
 	"os/user"
 	"path/filepath"
+	"slices"
 	"strconv"
 	"strings"
+	"syscall"
 	"testing"
 	"time"
 
-	"github.com/docker/docker/api/types"
 	"github.com/docker/docker/api/types/events"
+	"github.com/docker/docker/api/types/system"
 	"github.com/docker/docker/client"
 	"github.com/docker/docker/container"
 	"github.com/docker/docker/pkg/ioutils"
@@ -32,14 +38,14 @@ import (
 
 // LogT is the subset of the testing.TB interface used by the daemon.
 type LogT interface {
-	Logf(string, ...interface{})
+	Logf(string, ...any)
 }
 
 // nopLog is a no-op implementation of LogT that is used in daemons created by
 // NewDaemon (where no testing.TB is available).
 type nopLog struct{}
 
-func (nopLog) Logf(string, ...interface{}) {}
+func (nopLog) Logf(string, ...any) {}
 
 const (
 	defaultDockerdBinary         = "dockerd"
@@ -83,18 +89,22 @@ type Daemon struct {
 	args                       []string
 	extraEnv                   []string
 	containerdSocket           string
+	usernsRemap                string
 	rootlessUser               *user.User
 	rootlessXDGRuntimeDir      string
+	resolvConfContent          string
+	ResolvConfPathOverride     string // Path to a replacement for "/etc/resolv.conf", or empty.
 
 	// swarm related field
-	swarmListenAddr string
-	SwarmPort       int // FIXME(vdemeester) should probably not be exported
-	DefaultAddrPool []string
-	SubnetSize      uint32
-	DataPathPort    uint32
-	OOMScoreAdjust  int
+	swarmListenAddr   string
+	swarmWithIptables bool
+	SwarmPort         int // FIXME(vdemeester) should probably not be exported
+	DefaultAddrPool   []string
+	SubnetSize        uint32
+	DataPathPort      uint32
+	OOMScoreAdjust    int
 	// cached information
-	CachedInfo types.Info
+	CachedInfo system.Info
 }
 
 // NewDaemon returns a Daemon instance to be used for testing.
@@ -103,7 +113,7 @@ type Daemon struct {
 func NewDaemon(workingDir string, ops ...Option) (*Daemon, error) {
 	storageDriver := os.Getenv("DOCKER_GRAPHDRIVER")
 
-	if err := os.MkdirAll(SockRoot, 0700); err != nil {
+	if err := os.MkdirAll(SockRoot, 0o700); err != nil {
 		return nil, errors.Wrapf(err, "failed to create daemon socket root %q", SockRoot)
 	}
 
@@ -114,7 +124,7 @@ func NewDaemon(workingDir string, ops ...Option) (*Daemon, error) {
 		return nil, err
 	}
 	daemonRoot := filepath.Join(daemonFolder, "root")
-	if err := os.MkdirAll(daemonRoot, 0755); err != nil {
+	if err := os.MkdirAll(daemonRoot, 0o755); err != nil {
 		return nil, errors.Wrapf(err, "failed to create daemon root %q", daemonRoot)
 	}
 
@@ -143,8 +153,17 @@ func NewDaemon(workingDir string, ops ...Option) (*Daemon, error) {
 		op(d)
 	}
 
+	if len(d.resolvConfContent) > 0 {
+		path := filepath.Join(d.Folder, "resolv.conf")
+		if err := os.WriteFile(path, []byte(d.resolvConfContent), 0644); err != nil {
+			return nil, fmt.Errorf("failed to write docker resolv.conf to %q: %v", path, err)
+		}
+		d.extraEnv = append(d.extraEnv, "DOCKER_TEST_RESOLV_CONF_PATH="+path)
+		d.ResolvConfPathOverride = path
+	}
+
 	if d.rootlessUser != nil {
-		if err := os.Chmod(SockRoot, 0777); err != nil {
+		if err := os.Chmod(SockRoot, 0o777); err != nil {
 			return nil, err
 		}
 		uid, err := strconv.Atoi(d.rootlessUser.Uid)
@@ -161,20 +180,22 @@ func NewDaemon(workingDir string, ops ...Option) (*Daemon, error) {
 		if err := os.Chown(d.Root, uid, gid); err != nil {
 			return nil, err
 		}
-		if err := os.MkdirAll(filepath.Dir(d.execRoot), 0700); err != nil {
+		if err := os.MkdirAll(filepath.Dir(d.execRoot), 0o700); err != nil {
 			return nil, err
 		}
 		if err := os.Chown(filepath.Dir(d.execRoot), uid, gid); err != nil {
 			return nil, err
 		}
-		if err := os.MkdirAll(d.execRoot, 0700); err != nil {
+		if err := os.MkdirAll(d.execRoot, 0o700); err != nil {
 			return nil, err
 		}
 		if err := os.Chown(d.execRoot, uid, gid); err != nil {
 			return nil, err
 		}
-		d.rootlessXDGRuntimeDir = filepath.Join(d.Folder, "xdgrun")
-		if err := os.MkdirAll(d.rootlessXDGRuntimeDir, 0700); err != nil {
+		// $XDG_RUNTIME_DIR mustn't be too long, as ${XDG_RUNTIME_DIR/dockerd-rootless
+		// contains Unix sockets
+		d.rootlessXDGRuntimeDir = filepath.Join(os.TempDir(), "xdgrun-"+id)
+		if err := os.MkdirAll(d.rootlessXDGRuntimeDir, 0o700); err != nil {
 			return nil, err
 		}
 		if err := os.Chown(d.rootlessXDGRuntimeDir, uid, gid); err != nil {
@@ -213,6 +234,10 @@ func New(t testing.TB, ops ...Option) *Daemon {
 	}
 	ops = append(ops, WithOOMScoreAdjust(-500))
 
+	if val, ok := os.LookupEnv("DOCKER_FIREWALL_BACKEND"); ok {
+		ops = append(ops, WithEnvVars("DOCKER_FIREWALL_BACKEND="+val))
+	}
+
 	d, err := NewDaemon(dest, ops...)
 	assert.NilError(t, err, "could not create daemon at %q", dest)
 	if d.rootlessUser != nil && d.dockerdBinary != defaultDockerdBinary {
@@ -335,6 +360,17 @@ func ScanLogsMatchString(contains string) func(string) bool {
 	}
 }
 
+// ScanLogsMatchCount returns a function that can be used to scan the daemon logs until the passed in matcher function matches `count` times
+func ScanLogsMatchCount(f func(string) bool, count int) func(string) bool {
+	matched := 0
+	return func(line string) bool {
+		if f(line) {
+			matched++
+		}
+		return matched == count
+	}
+}
+
 // ScanLogsMatchAll returns a function that can be used to scan the daemon logs until *all* the passed in strings are matched
 func ScanLogsMatchAll(contains ...string) func(string) bool {
 	matched := make(map[string]bool)
@@ -408,7 +444,7 @@ func (d *Daemon) Start(t testing.TB, args ...string) {
 // StartWithError starts the daemon and return once it is ready to receive requests.
 // It returns an error in case it couldn't start.
 func (d *Daemon) StartWithError(args ...string) error {
-	logFile, err := os.OpenFile(filepath.Join(d.Folder, "docker.log"), os.O_RDWR|os.O_CREATE|os.O_APPEND, 0600)
+	logFile, err := os.OpenFile(filepath.Join(d.Folder, "docker.log"), os.O_RDWR|os.O_CREATE|os.O_APPEND, 0o600)
 	if err != nil {
 		return errors.Wrapf(err, "[%s] failed to create logfile", d.id)
 	}
@@ -446,6 +482,8 @@ func (d *Daemon) StartWithLogFile(out *os.File, providedArgs ...string) error {
 	}
 
 	d.args = append(d.args,
+		// Make sure we don't use the environment-provided global config file.
+		"--config-file", "/dev/null",
 		"--data-root", d.Root,
 		"--exec-root", d.execRoot,
 		"--pidfile", d.pidFile,
@@ -457,6 +495,10 @@ func (d *Daemon) StartWithLogFile(out *os.File, providedArgs ...string) error {
 		d.args = append(d.args, "--containerd", d.containerdSocket)
 	}
 
+	if d.usernsRemap != "" {
+		d.args = append(d.args, "--userns-remap", d.usernsRemap)
+	}
+
 	if d.defaultCgroupNamespaceMode != "" {
 		d.args = append(d.args, "--default-cgroupns-mode", d.defaultCgroupNamespaceMode)
 	}
@@ -466,7 +508,7 @@ func (d *Daemon) StartWithLogFile(out *os.File, providedArgs ...string) error {
 	if d.init {
 		d.args = append(d.args, "--init")
 	}
-	if !(d.UseDefaultHost || d.UseDefaultTLSHost) {
+	if !d.UseDefaultHost && !d.UseDefaultTLSHost {
 		d.args = append(d.args, "--host", d.Sock())
 	}
 	if root := os.Getenv("DOCKER_REMAP_ROOT"); root != "" {
@@ -496,6 +538,7 @@ func (d *Daemon) StartWithLogFile(out *os.File, providedArgs ...string) error {
 	cmd := exec.Command(dockerdBinary, d.args...)
 	cmd.Env = append(os.Environ(), "DOCKER_SERVICE_PREFER_OFFLINE_IMAGE=1")
 	cmd.Env = append(cmd.Env, d.extraEnv...)
+	cmd.Env = append(cmd.Env, "OTEL_SERVICE_NAME=dockerd-"+d.id)
 	cmd.Stdout = out
 	cmd.Stderr = out
 	d.logFile = out
@@ -581,10 +624,10 @@ func (d *Daemon) StartWithLogFile(out *os.File, providedArgs ...string) error {
 
 // StartWithBusybox will first start the daemon with Daemon.Start()
 // then save the busybox image from the main daemon and load it into this Daemon instance.
-func (d *Daemon) StartWithBusybox(t testing.TB, arg ...string) {
+func (d *Daemon) StartWithBusybox(ctx context.Context, t testing.TB, arg ...string) {
 	t.Helper()
 	d.Start(t, arg...)
-	d.LoadBusybox(t)
+	d.LoadBusybox(ctx, t)
 }
 
 // Kill will send a SIGKILL to the daemon
@@ -602,6 +645,11 @@ func (d *Daemon) Kill() error {
 		return err
 	}
 
+	_, err := d.cmd.Process.Wait()
+	if err != nil && !errors.Is(err, syscall.ECHILD) {
+		return err
+	}
+
 	if d.pidFile != "" {
 		_ = os.Remove(d.pidFile)
 	}
@@ -657,13 +705,13 @@ func (d *Daemon) Stop(t testing.TB) {
 // If it timeouts, a SIGKILL is sent.
 // Stop will not delete the daemon directory. If a purged daemon is needed,
 // instantiate a new one with NewDaemon.
-func (d *Daemon) StopWithError() (err error) {
+func (d *Daemon) StopWithError() (retErr error) {
 	if d.cmd == nil || d.Wait == nil {
 		return errDaemonNotStarted
 	}
 	defer func() {
-		if err != nil {
-			d.log.Logf("[%s] error while stopping daemon: %v", d.id, err)
+		if retErr != nil {
+			d.log.Logf("[%s] error while stopping daemon: %v", d.id, retErr)
 		} else {
 			d.log.Logf("[%s] daemon stopped", d.id)
 			if d.pidFile != "" {
@@ -764,7 +812,7 @@ func (d *Daemon) ReloadConfig() error {
 	errCh := make(chan error, 1)
 	started := make(chan struct{})
 	go func() {
-		_, body, err := request.Get("/events", request.Host(d.Sock()))
+		_, body, err := request.Get(context.TODO(), "/events", request.Host(d.Sock()))
 		close(started)
 		if err != nil {
 			errCh <- err
@@ -781,7 +829,7 @@ func (d *Daemon) ReloadConfig() error {
 			if e.Type != events.DaemonEventType {
 				continue
 			}
-			if e.Action != "reload" {
+			if e.Action != events.ActionReload {
 				continue
 			}
 			close(errCh) // notify that we are done
@@ -804,23 +852,37 @@ func (d *Daemon) ReloadConfig() error {
 	return nil
 }
 
+// SetEnvVar updates the set of extra env variables for the daemon, to take
+// effect on the next start/restart.
+func (d *Daemon) SetEnvVar(name, val string) {
+	prefix := name + "="
+	if idx := slices.IndexFunc(d.extraEnv, func(ev string) bool { return strings.HasPrefix(ev, prefix) }); idx > 0 {
+		d.extraEnv[idx] = prefix + val
+		return
+	}
+	d.extraEnv = append(d.extraEnv, prefix+val)
+}
+
 // LoadBusybox image into the daemon
-func (d *Daemon) LoadBusybox(t testing.TB) {
+func (d *Daemon) LoadBusybox(ctx context.Context, t testing.TB) {
+	d.LoadImage(ctx, t, "busybox:latest")
+}
+
+func (d *Daemon) LoadImage(ctx context.Context, t testing.TB, img string) {
 	t.Helper()
 	clientHost, err := client.NewClientWithOpts(client.FromEnv)
 	assert.NilError(t, err, "[%s] failed to create client", d.id)
 	defer clientHost.Close()
 
-	ctx := context.Background()
-	reader, err := clientHost.ImageSave(ctx, []string{"busybox:latest"})
-	assert.NilError(t, err, "[%s] failed to download busybox", d.id)
+	reader, err := clientHost.ImageSave(ctx, []string{img})
+	assert.NilError(t, err, "[%s] failed to download %s", d.id, img)
 	defer reader.Close()
 
 	c := d.NewClientT(t)
 	defer c.Close()
 
-	resp, err := c.ImageLoad(ctx, reader, true)
-	assert.NilError(t, err, "[%s] failed to load busybox", d.id)
+	resp, err := c.ImageLoad(ctx, reader, client.ImageLoadWithQuiet(true))
+	assert.NilError(t, err, "[%s] failed to load %s", d.id, img)
 	defer resp.Body.Close()
 }
 
@@ -917,7 +979,7 @@ func (d *Daemon) queryRootDir() (string, error) {
 }
 
 // Info returns the info struct for this daemon
-func (d *Daemon) Info(t testing.TB) types.Info {
+func (d *Daemon) Info(t testing.TB) system.Info {
 	t.Helper()
 	c := d.NewClientT(t)
 	info, err := c.Info(context.Background())
@@ -926,6 +988,29 @@ func (d *Daemon) Info(t testing.TB) types.Info {
 	return info
 }
 
+func (d *Daemon) FirewallBackendDriver(t testing.TB) string {
+	t.Helper()
+	info := d.Info(t)
+	assert.Assert(t, info.FirewallBackend != nil, "no firewall backend reported")
+	return info.FirewallBackend.Driver
+}
+
+// FirewallReloadedAt fetches the daemon's Info and, if it contains a firewall
+// reload time, returns that time.
+func (d *Daemon) FirewallReloadedAt(t testing.TB) string {
+	t.Helper()
+	info := d.Info(t)
+	if info.FirewallBackend == nil {
+		return ""
+	}
+	for _, kv := range info.FirewallBackend.Info {
+		if kv[0] == "ReloadedAt" {
+			return kv[1]
+		}
+	}
+	return ""
+}
+
 // TamperWithContainerConfig modifies the on-disk config of a container.
 func (d *Daemon) TamperWithContainerConfig(t testing.TB, containerID string, tamper func(*container.Container)) {
 	t.Helper()
@@ -940,7 +1025,7 @@ func (d *Daemon) TamperWithContainerConfig(t testing.TB, containerID string, tam
 	tamper(&c)
 	configBytes, err = json.Marshal(&c)
 	assert.NilError(t, err)
-	assert.NilError(t, os.WriteFile(configPath, configBytes, 0600))
+	assert.NilError(t, os.WriteFile(configPath, configBytes, 0o600))
 }
 
 // cleanupRaftDir removes swarmkit wal files if present
